import { AIConfigParser, type AIConfig } from '@/provider/utils/parse-config.ts';
import { redis } from '@/modules/db.ts';
import { CustomError } from '@kevisual/router';
import { queryConfig } from '@/modules/query.ts';
import { logger } from '@/modules/logger.ts';
export class ChatConfigServices {
  cachePrefix = 'ai:chat:config';
  // 使用谁的模型
  owner: string;
  // 使用者
  username: string;
  aiConfig?: AIConfig;
  isOwner: boolean;
  token?: string;
  /**
   * username 是使用的模型的用户名，使用谁的模型
   * @param username
   */
  constructor(owner: string, username: string, token?: string) {
    this.owner = owner;
    this.username = username;
    this.isOwner = owner === username;
    // this.token = token;
  }
  getKey() {
    return `${this.cachePrefix}:${this.owner}`;
  }
  /**
   * 获取chat配置
   * @param keepSecret 是否需要清除secret 默认 不清除 为true
   * @returns
   */
  async getChatConfig(keepSecret = true, newToken?: string) {
    const key = this.getKey();
    const cache = await redis.get(key);
    let modelConfig = null;
    if (cache) {
      modelConfig = JSON.parse(cache);
    }
    const token = newToken || this.token;
    if (!modelConfig) {
      if (this.owner !== this.username) {
        throw new CustomError(
          `the owner [${this.owner}] config, [${this.username}] not permission to init config, only owner can init config, place connect owner`,
        );
      } else {
        const res = await queryConfig.getConfigByKey('ai.json', { token });
        if (res.code === 200 && res.data?.data) {
          modelConfig = res.data.data;
        } else {
          logger.error('获取ai.json配置失败', res, 'username', this.username);
          throw new CustomError(400, 'get config failed');
        }
      }
    }
    if (!modelConfig) {
      throw new CustomError(`${this.owner} modelConfig is null`);
    }
    if (!cache) {
      const cacheTime = 60 * 60 * 24 * 40; // 1天
      await redis.set(key, JSON.stringify(modelConfig), 'EX', cacheTime);
    }
    this.aiConfig = modelConfig;
    const aiConfigParser = new AIConfigParser(modelConfig);
    return aiConfigParser.getConfig(keepSecret);
  }
  async clearCache() {
    const key = this.getKey();
    await redis.set(key, JSON.stringify({}), 'EX', 1);
  }
  /**
   * 获取模型配置
   * @returns
   */
  async getSelectOpts(config?: AIConfig) {
    const aiConfigParser = new AIConfigParser(config || this.aiConfig);
    return aiConfigParser.getSelectOpts();
  }
  /**
   * 获取和检测当前用户的额度, 当使用 root 账号的时候，才需要检测
   * username是当前使用用户
   * @param username
   */
  async checkUserCanChat(username: string) {
    if (this.owner !== 'root') return true;
    const maxToken = 100000;
    const userCacheKey = `${this.cachePrefix}:root:chat-limit:${username}`;
    const cache = await redis.get(userCacheKey);
    if (cache) {
      const cacheData = JSON.parse(cache);
      if (cacheData.token >= maxToken) {
        throw new CustomError(400, 'use root account token limit exceeded');
      }
    }
    return true;
  }
  /**
   * 获取用户的使用情况
   * username是当前使用用户
   * @param username
   * @returns
   */
  async getUserChatLimit(username: string) {
    if (this.owner !== 'root') return;
    const userCacheKey = `${this.cachePrefix}:root:chat-limit:${username}`;
    const cache = await redis.get(userCacheKey);
    if (cache) {
      const cacheData = JSON.parse(cache);
      return cacheData;
    }
    return {
      token: 0,
      day: 0,
    };
  }

  /**
   * 更新用户的使用情况
   * username是当前使用用户
   * @param username
   * @param token
   */
  async updateUserChatLimit(username: string, token: number) {
    if (this.owner !== 'root') return;
    const userCacheKey = `${this.cachePrefix}:root:chat-limit:${username}`;
    const cache = await redis.get(userCacheKey);
    if (cache) {
      const cacheData = JSON.parse(cache);
      cacheData.token = cacheData.token + token;
      await redis.set(userCacheKey, JSON.stringify(cacheData), 'EX', 60 * 60 * 24 * 30); // 30天
    } else {
      await redis.set(userCacheKey, JSON.stringify({ token }), 'EX', 60 * 60 * 24 * 30); // 30天
    }
  }
  async clearChatLimit() {
    if (this.owner !== 'root') return;
    // const userCacheKey = `${this.cachePrefix}:root:chat-limit:${this.username}`;
    // await redis.del(userCacheKey);
  }
}
